﻿using System.Windows.Forms;

namespace AirBomber
{
    public partial class FormMapWithSetAirplanes : Form
    {
        /// <summary>
        /// Словарь для выпадающего списка
        /// </summary>
        private readonly Dictionary<string, AbstractMap> _mapsDict = new()
        {
            { "Простая карта", new SimpleMap() },
            { "Карта со стенами", new WallMap() },
        };
        /// <summary>
        /// Объект от коллекции карт
        /// </summary>
        private readonly MapsCollection _mapsCollection;
        private GeneratorAirplane<EntityAirplane, IAirplaneEngines> _generatorAirplane;
        /// <summary>
        /// Все объекты удаленные с каких либо карт
        /// </summary>
        private LinkedList<DrawningObject> _deletedObjects;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormMapWithSetAirplanes()
        {
            InitializeComponent();
            _generatorAirplane = new(100, 100);
            _mapsCollection = new MapsCollection(pictureBox.Width, pictureBox.Height);
            _deletedObjects = new();
            comboBoxSelectorMap.Items.Clear();
            foreach (var elem in _mapsDict)
            {
                comboBoxSelectorMap.Items.Add(elem.Key);
            }
        }

        private string NameMap => listBoxMaps.SelectedItem?.ToString() ?? string.Empty;
        /// <summary>
        /// Заполнение listBoxMaps
        /// </summary>
        private void ReloadMaps()
        {
            int index = listBoxMaps.SelectedIndex;

            listBoxMaps.Items.Clear();
            for (int i = 0; i < _mapsCollection.Keys.Count; i++)
            {
                listBoxMaps.Items.Add(_mapsCollection.Keys[i]);
            }

            if (listBoxMaps.Items.Count > 0 && (index == -1 || index >= listBoxMaps.Items.Count))
            {
                listBoxMaps.SelectedIndex = 0;
            }
            else if (listBoxMaps.Items.Count > 0 && index > -1 && index < listBoxMaps.Items.Count)
            {
                listBoxMaps.SelectedIndex = index;
            }
        }
        /// <summary>
        /// Добавление карты
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddMap_Click(object sender, EventArgs e)
        {
            if (comboBoxSelectorMap.SelectedIndex == -1 || string.IsNullOrEmpty(textBoxNewMapName.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (!_mapsDict.ContainsKey(comboBoxSelectorMap.Text))
            {
                MessageBox.Show("Нет такой карты", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _mapsCollection.AddMap(textBoxNewMapName.Text, _mapsDict[comboBoxSelectorMap.Text]);
            ReloadMaps();
        }
        /// <summary>
        /// Выбор карты
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ListBoxMaps_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBox.Image = _mapsCollection[NameMap].ShowSet();
        }
        /// <summary>
        /// Удаление карты
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonDeleteMap_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }

            if (MessageBox.Show($"Удалить карту {listBoxMaps.SelectedItem}?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _mapsCollection.DelMap(NameMap);
                ReloadMaps();
            }
        }
        /// <summary>Добавление самолета на карту</summary>
        /// <param name="airplane">Самолет</param>
        /// <param name="text">Текст MessageBox если самолета нет</param>
        /// <param name="caption">Заголовок MessageBox если самолета нет</param>
        private void AddAirplaneInMap(DrawningObject? airplane, string? text = null, string? caption = null)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                MessageBox.Show("Перед добавлением объекта необходимо создать карту");
            }
            else if (airplane == null)
            {
                MessageBox.Show(text, caption);
            }
            else if ((_mapsCollection[NameMap] + airplane) == -1)
            {
                MessageBox.Show("Не удалось добавить объект");
            }
            else
            {
                MessageBox.Show("Объект добавлен");
                pictureBox.Image = _mapsCollection[NameMap].ShowSet();
            }
        }
        /// <summary>
        /// Перемещение
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            //получаем имя кнопки
            string name = ((Button)sender)?.Name ?? string.Empty;
            Direction dir = Direction.None;
            switch (name)
            {
                case "buttonUp":
                    dir = Direction.Up;
                    break;
                case "buttonDown":
                    dir = Direction.Down;
                    break;
                case "buttonLeft":
                    dir = Direction.Left;
                    break;
                case "buttonRight":
                    dir = Direction.Right;
                    break;
            }
            pictureBox.Image = _mapsCollection[NameMap].MoveObject(dir);
        }
        /// <summary>
        /// Добавления типа двигателя и его количество в генератор
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonAddTypeOfEntity_Click(object sender, EventArgs e)
        {
            Random rnd = new();
            Color colorBody = Color.FromArgb(rnd.Next() % 256, rnd.Next() % 256, rnd.Next() % 256);
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                colorBody = dialog.Color;
            }
            var entity = new EntityAirplane((int)numericSpeed.Value, (int)numericWeight.Value, colorBody);
            _generatorAirplane.AddTypeOfEntity(entity);
            MessageBox.Show($"Добавлены свойства самолета:\n" +
                $"Вес: {entity.Weight}\n" +
                $"Скорость: {entity.Speed}\n" +
                $"Цвет: {colorBody.Name}", 
                "Успешно добавлены свойства");
        }
        /// <summary>
        /// Генерация самолета
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnGenerateAirplane_Click(object sender, EventArgs e)
        {
            AddAirplaneInMap(_generatorAirplane.Generate(), 
                "Не удалось сгенерировать самолет. Добавьте хотя бы по одному количество двигателей и свойств для генерации", 
                "Генерация самолета"
                );
        }
        /// <summary>
        /// Добавления сущности в генератор
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnAddTypeOfEngines_Click(object sender, EventArgs e)
        {
            IAirplaneEngines? typeAirplaneEngines = null;
            switch (comboTypeEngines.Text)
            {
                case "Квадратный":
                    typeAirplaneEngines = new AirplaneRectEngines();
                    break;
                case "Стрелка":
                    typeAirplaneEngines = new AirplaneArrowEngines();
                    break;
                default:
                    typeAirplaneEngines = new DrawningAirplaneEngines();
                    break;
            }
            typeAirplaneEngines.CountEngines = (int)numericUpDownEngines.Value;
            _generatorAirplane.AddTypeOfEngines(typeAirplaneEngines);
        }
        /// <summary>
        /// Вызов формы для создания самолета
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddAirplane_Click(object sender, EventArgs e)
        {
            FormAirplaneConfig formAirplane = new();
            formAirplane.AddEvent(new(AddAirplane));
            formAirplane.Show();
        }
        /// <summary>
        /// Добавление объекта
        /// </summary>
        /// <param name="airplane"></param>
        private void AddAirplane(DrawningAirplane airplane)
        {
            AddAirplaneInMap(new(airplane));
        }
        /// <summary>
        /// Удаление объекта
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRemoveAirplane_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1 || string.IsNullOrEmpty(maskedTextBoxPosition.Text)
                || MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
            var deletedObject = _mapsCollection[NameMap, pos];
            if (_mapsCollection[NameMap] - pos != null)
            {
                MessageBox.Show("Объект удален");
                _deletedObjects.AddLast(deletedObject);
                pictureBox.Image = _mapsCollection[NameMap].ShowSet();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }
        /// <summary>
        /// Вывод набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonShowStorage_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            pictureBox.Image = _mapsCollection[NameMap].ShowSet();
        }
        /// <summary>
        /// Вывод карты
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonShowOnMap_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            pictureBox.Image = _mapsCollection[NameMap].ShowOnMap();
        }

        private void btnAddDeletedObject_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            if (_deletedObjects.Count == 0)
            {
                MessageBox.Show("Нет удаленных объектов", "Добавление удаленного объекта");
                return;
            }
            FormAirBomber form = new((DrawningAirplane)_deletedObjects.First());
            _deletedObjects.RemoveFirst();
            if (form.ShowDialog() == DialogResult.OK)
            {
                AddAirplaneInMap(form.SelectedAirplane);
            }
        }
    }
}